@@ -1,5 +1,6 @@
 //! This module contains all DICOM data element encoding logic.
 use byteordered::Endianness;
+use dicom_core::value::serialize::{encode_date, encode_datetime, encode_time};
 use dicom_core::{DataElementHeader, PrimitiveValue, Tag};
 use snafu::{Backtrace, ResultExt, Snafu};
 use std::fmt;
@@ -11,9 +12,6 @@ pub mod explicit_be;
 pub mod explicit_le;
 pub mod implicit_le;
 
-#[deprecated]
-pub use dicom_core::value::serialize as primitive_value;
-
 /// Module-level error type:
 /// for errors which may occur while encoding DICOM data.
 #[derive(Debug, Snafu)]
@@ -85,9 +83,14 @@ pub enum Error {
         backtrace: Backtrace,
         source: io::Error,
     },
+    #[snafu(display("Failed to write pixel data offset table"))]
+    WriteOffsetTable {
+        backtrace: Backtrace,
+        source: io::Error,
+    },
 }
 
-pub type Result<T> = std::result::Result<T, Error>;
+pub type Result<T, E = Error> = std::result::Result<T, E>;
 
 /// Type trait for an encoder of basic data properties.
 /// Unlike `Encode` (and similar to `BasicDecode`), this trait is not object
@@ -159,25 +162,25 @@ pub trait BasicEncode {
         use PrimitiveValue::*;
         match value {
             Empty => Ok(0), // no-op
-            Date(date) => encode_collection_delimited(&mut to, &*date, |to, date| {
-                primitive_value::encode_date(to, *date)
-            })
-            .context(WriteDate),
-            Time(time) => encode_collection_delimited(&mut to, &*time, |to, time| {
-                primitive_value::encode_time(to, *time)
-            })
-            .context(WriteTime),
+            Date(date) => {
+                encode_collection_delimited(&mut to, &*date, |to, date| encode_date(to, *date))
+                    .context(WriteDateSnafu)
+            }
+            Time(time) => {
+                encode_collection_delimited(&mut to, &*time, |to, time| encode_time(to, *time))
+                    .context(WriteTimeSnafu)
+            }
             DateTime(datetime) => {
                 encode_collection_delimited(&mut to, &*datetime, |to, datetime| {
-                    primitive_value::encode_datetime(to, *datetime)
+                    encode_datetime(to, *datetime)
                 })
-                .context(WriteDateTime)
+                .context(WriteDateTimeSnafu)
             }
             Str(s) => {
                 // Note: this will always print in UTF-8. Consumers should
                 // intercept string primitive values and encode them according
                 // to the expected character set.
-                write!(to, "{}", s).context(WriteString)?;
+                write!(to, "{}", s).context(WriteStringSnafu)?;
                 Ok(s.len())
             }
             Strs(s) => encode_collection_delimited(&mut to, &*s, |to, s| {
@@ -187,71 +190,72 @@ pub trait BasicEncode {
                 write!(to, "{}", s)?;
                 Ok(s.len())
             })
-            .context(WriteString),
+            .context(WriteStringSnafu),
             F32(values) => {
                 for v in values {
                     self.encode_fl(&mut to, *v)
-                        .context(WriteBinary { typ: "F32" })?;
+                        .context(WriteBinarySnafu { typ: "F32" })?;
                 }
                 Ok(values.len() * 4)
             }
             F64(values) => {
                 for v in values {
                     self.encode_fd(&mut to, *v)
-                        .context(WriteBinary { typ: "F64" })?;
+                        .context(WriteBinarySnafu { typ: "F64" })?;
                 }
                 Ok(values.len() * 8)
             }
             U64(values) => {
                 for v in values {
                     self.encode_uv(&mut to, *v)
-                        .context(WriteBinary { typ: "U64" })?;
+                        .context(WriteBinarySnafu { typ: "U64" })?;
                 }
                 Ok(values.len() * 8)
             }
             I64(values) => {
                 for v in values {
                     self.encode_sv(&mut to, *v)
-                        .context(WriteBinary { typ: "I64" })?;
+                        .context(WriteBinarySnafu { typ: "I64" })?;
                 }
                 Ok(values.len() * 8)
             }
             U32(values) => {
                 for v in values {
                     self.encode_ul(&mut to, *v)
-                        .context(WriteBinary { typ: "U32" })?;
+                        .context(WriteBinarySnafu { typ: "U32" })?;
                 }
                 Ok(values.len() * 4)
             }
             I32(values) => {
                 for v in values {
                     self.encode_sl(&mut to, *v)
-                        .context(WriteBinary { typ: "I32" })?;
+                        .context(WriteBinarySnafu { typ: "I32" })?;
                 }
                 Ok(values.len() * 4)
             }
             U16(values) => {
                 for v in values {
                     self.encode_us(&mut to, *v)
-                        .context(WriteBinary { typ: "U16" })?;
+                        .context(WriteBinarySnafu { typ: "U16" })?;
                 }
                 Ok(values.len() * 2)
             }
             I16(values) => {
                 for v in values {
                     self.encode_ss(&mut to, *v)
-                        .context(WriteBinary { typ: "I16" })?;
+                        .context(WriteBinarySnafu { typ: "I16" })?;
                 }
                 Ok(values.len() * 2)
             }
             U8(values) => {
-                to.write_all(values).context(WriteBytes)?;
+                to.write_all(values).context(WriteBytesSnafu)?;
                 Ok(values.len())
             }
             Tags(tags) => {
                 for tag in tags {
-                    self.encode_us(&mut to, tag.0).context(WriteTagGroup)?;
-                    self.encode_us(&mut to, tag.1).context(WriteTagElement)?;
+                    self.encode_us(&mut to, tag.0).context(WriteTagGroupSnafu)?;
+                    self.encode_us(&mut to, tag.1)
+                        .context(WriteTagElementSnafu)?;
                 }
                 Ok(tags.len() * 4)
             }
@@ -305,7 +309,7 @@ pub trait Encode {
         W: Write,
     {
         self.encode_tag(&mut to, Tag(0xFFFE, 0xE00D))?;
-        to.write_all(&[0u8; 4]).context(WriteItemDelimiter)?;
+        to.write_all(&[0u8; 4]).context(WriteItemDelimiterSnafu)?;
         Ok(())
     }
 
@@ -315,7 +319,8 @@ pub trait Encode {
         W: Write,
     {
         self.encode_tag(&mut to, Tag(0xFFFE, 0xE0DD))?;
-        to.write_all(&[0u8; 4]).context(WriteSequenceDelimiter)?;
+        to.write_all(&[0u8; 4])
+            .context(WriteSequenceDelimiterSnafu)?;
         Ok(())
     }
 
@@ -323,6 +328,17 @@ pub trait Encode {
     fn encode_primitive<W>(&self, to: W, value: &PrimitiveValue) -> Result<usize>
     where
         W: Write;
+
+    /// Encode and write a DICOM pixel data offset table
+    /// to the given destination.
+    ///
+    /// For convenience, returns the number of bytes written on success,
+    /// equivalent to `offset_table.len() * 4`.
+    // Note that offset tables might not apply to all forms of encoding,
+    // but this method needs to exist nevertheless.
+    fn encode_offset_table<W>(&self, to: W, offset_table: &[u32]) -> Result<usize>
+    where
+        W: Write;
 }
 
 impl<T: ?Sized> Encode for &T
@@ -370,6 +386,13 @@ where
     {
         (**self).encode_primitive(to, value)
     }
+
+    fn encode_offset_table<W>(&self, to: W, offset_table: &[u32]) -> Result<usize>
+    where
+        W: Write,
+    {
+        (**self).encode_offset_table(to, offset_table)
+    }
 }
 
 impl<T: ?Sized> Encode for Box<T>
@@ -417,6 +440,13 @@ where
     {
         (**self).encode_primitive(to, value)
     }
+
+    fn encode_offset_table<W>(&self, to: W, offset_table: &[u32]) -> Result<usize>
+    where
+        W: Write,
+    {
+        (**self).encode_offset_table(to, offset_table)
+    }
 }
 
 /// Type trait for a data element encoder to a single known writer type `W`.
@@ -428,6 +458,10 @@ pub trait EncodeTo<W: ?Sized> {
 
     /// Encode and write a data element header to the given destination.
     /// Returns the number of bytes effectively written on success.
+    ///
+    /// Note that data element header should be encoded as is,
+    /// regardless of the given value length.
+    /// Any eventual padding logic should be done at a higher level.
     fn encode_element_header(&self, to: &mut W, de: DataElementHeader) -> Result<usize>
     where
         W: Write;
@@ -453,6 +487,17 @@ pub trait EncodeTo<W: ?Sized> {
     fn encode_primitive(&self, to: &mut W, value: &PrimitiveValue) -> Result<usize>
     where
         W: Write;
+
+    /// Encode and write a DICOM pixel data offset table
+    /// to the given destination.
+    ///
+    /// For convenience, returns the number of bytes written on success,
+    /// equivalent to `offset_table.len() * 4`.
+    // Note that offset tables might not apply to all forms of encoding,
+    // but this method needs to exist nevertheless.
+    fn encode_offset_table(&self, to: &mut W, offset_table: &[u32]) -> Result<usize>
+    where
+        W: Write;
 }
 
 impl<T, W: ?Sized> EncodeTo<W> for &T
@@ -501,6 +546,13 @@ where
     {
         (**self).encode_primitive(to, value)
     }
+
+    fn encode_offset_table(&self, to: &mut W, offset_table: &[u32]) -> Result<usize>
+    where
+        W: Write,
+    {
+        (**self).encode_offset_table(to, offset_table)
+    }
 }
 
 impl<T: ?Sized, W: ?Sized> EncodeTo<W> for Box<T>
@@ -549,6 +601,13 @@ where
     {
         (**self).encode_primitive(to, value)
     }
+
+    fn encode_offset_table(&self, to: &mut W, offset_table: &[u32]) -> Result<usize>
+    where
+        W: Write,
+    {
+        (**self).encode_offset_table(to, offset_table)
+    }
 }
 
 /// A type binding of an encoder to a target writer.
@@ -684,6 +743,10 @@ where
     fn encode_primitive(&self, to: &mut W, value: &PrimitiveValue) -> Result<usize> {
         self.inner.encode_primitive(to, value)
     }
+
+    fn encode_offset_table(&self, to: &mut W, offset_table: &[u32]) -> Result<usize> {
+        self.inner.encode_offset_table(to, offset_table)
+    }
 }
 
 #[cfg(test)]
