@@ -15,12 +15,18 @@
 //!
 //! [transfer syntax specifier]: ./transfer_syntax/index.html
 
+pub mod adapters;
 pub mod decode;
 pub mod encode;
 pub mod text;
 pub mod transfer_syntax;
 
+pub use adapters::NeverPixelAdapter;
+pub use byteordered::Endianness;
 pub use decode::Decode;
 pub use encode::Encode;
 pub use transfer_syntax::Codec;
+pub use transfer_syntax::DataRWAdapter;
+pub use transfer_syntax::NeverAdapter;
 pub use transfer_syntax::TransferSyntax;
+pub use transfer_syntax::TransferSyntaxIndex;
